@@ -11,26 +11,22 @@ from apgw.types import (ConnectionArgs, InsertAssignments, LimitOffset, Record,
                         RecordsAndCount)
 
 
-class DB:
+class DBBase:
     """A simple wrapper around asyncpg to alleviate some boilerplate."""
 
     def __init__(
         self,
-        conn_args: ConnectionArgs,
         *,
-        auto_open: bool = False,
         logger: Logger = getLogger("apgw"),
-        _connect: Callable[..., Connection] = connect,
+        connect: Callable[..., Connection] = connect,
     ) -> None:
         """Initializes the DB instance.
 
-        :param conn_args: the connection arguments
         :param logger: a Logger instance
+        :param connection: method to connect to the database
         """
-        self.conn_args = conn_args
-        self._auto_open = auto_open
         self.logger = logger
-        self._connect = _connect
+        self.connect = connect
         self._conn: Optional[Connection] = None
 
     @property
@@ -45,27 +41,32 @@ class DB:
         return self._conn
 
     @property
-    def _should_auto_open(self) -> bool:
+    def should_auto_open(self) -> bool:
         """Returns True if the connection can and should be opened automatically."""
-        return self._auto_open and not self.is_open
+        # return self._auto_open and not self.is_open
+        return True
 
     async def auto_open(self) -> bool:
         """Opens the database if it should auto open."""
-        if self._should_auto_open:
+        if self.should_auto_open:
             await self.open()
             return True
         return False
 
-    async def open(self) -> "DB":
+    async def get_connection_args(self) -> ConnectionArgs:
+        raise NotImplementedError()
+
+    async def open(self) -> "DBBase":
         """Opens the connection."""
         if not self._conn:
-            if isinstance(self.conn_args, str):
-                self._conn = await self._connect(dsn=self.conn_args)
+            conn_args = await self.get_connection_args()
+            if isinstance(conn_args, str):
+                self._conn = await self.connect(dsn=conn_args)
             else:
-                self._conn = await self._connect(**self.conn_args)
+                self._conn = await self.connect(**conn_args)
         return self
 
-    async def close(self) -> "DB":
+    async def close(self) -> "DBBase":
         """Closes the connection."""
         if self._conn:
             await self._conn.close()
@@ -73,13 +74,13 @@ class DB:
         return self
 
     @asynccontextmanager
-    async def transaction(self) -> AsyncIterator["DB"]:
+    async def transaction(self) -> AsyncIterator["DBBase"]:
         """Run a command in a database transaction."""
         await self.auto_open()
         async with self.conn.transaction():
             yield self
 
-    async def __aenter__(self) -> "DB":
+    async def __aenter__(self) -> "DBBase":
         """Asynchronous context manager entry point."""
         await self.open()
         return self
@@ -311,3 +312,31 @@ class DB:
         (sql, params) = builder.delete_sql(table_name, constraints, returning=returning)
 
         return await self.fetch(sql, params)
+
+
+class DB(DBBase):
+    """A DBBase implementation that has static connection args."""
+    def __init__(
+        self,
+        conn_args: ConnectionArgs,
+        *,
+        auto_open: bool = False,
+        logger: Logger = getLogger("apgw"),
+        connect: Callable[..., Connection] = connect,
+    ) -> None:
+        """Initializes the DB instance.
+
+        :param conn_args: the connection arguments
+        :param auto_open: if the connection should automatically open
+        :param logger: a Logger instance
+        """
+        super().__init__(logger=logger, connect=connect)
+        self.conn_args = conn_args
+        self._auto_open = auto_open
+
+    @property
+    def should_auto_open(self) -> bool:
+        return self._auto_open
+
+    async def get_connection_args(self) -> Connection:
+        return self.conn_args
